module Rip
  module Helpers
    def warn(msg)
      $stderr.puts(msg)
    end

    def info(msg)
      warn(msg) if ENV['RIPVERBOSE'] || ENV['RIPDEBUG']
    end

    def debug(msg)
      warn(msg) if ENV['RIPDEBUG']
    end

    def metadata(package)
      if data = rip("metadata #{package}")
        Package.parse(data)
      end
    end

    def escape(*args)
      Escape.shell_command(args.flatten.compact)
    end

    def write(file, &content)
      File.open(file, 'w') do |f|
        text = content.call
        f.puts text.respond_to?(:join) ? text.join("\n") : text
      end
    end

    def read(file)
      File.read(file)
    end

    def basename(file)
      File.basename(file)
    end

    def commands
      ENV['RIPCOMMANDSCACHE'] ||= rip(:commands)
    end

    def rip(command, *args, &block)
      if block_given?
        args << "r"
        rip_io(command, *args) do |io|
          io.each_line do |line|
            block.call(line.chomp)
          end
        end
      else
        args = escape(args)
        debug "rip-#{command} #{args}"
        sh "rip-#{command}", args
      end
    end

    def rip_io(command, *args, &block)
      mode = args.pop
      args = escape(args)

      debug "rip-#{command} #{args}"
      result = IO.popen("rip-#{command} #{args}", mode, &block)

      if $?.success?
        result
      elsif $-e
        exit 1
      else
        nil
      end
    end

    def rpg_available?
      return false if ENV['RIPRPG'].nil? or ENV['RIPRPG'] == '0'
      `which rpg`
      $?.success?
    end

    def gem(command, *args)
      args = escape(args) + " 2> /dev/null"
      args << " -s #{ENV["GEM_SERVER"]}" if ENV["GEM_SERVER"]
      args << " 2> /dev/null"

      debug "gem #{command} #{args}"
      `gem #{command} #{args}`
    end

    def rpg(command, *args)
      args = escape(args)
      args << " 2> /dev/null"

      debug "rpg #{command} #{args}"
      `rpg #{command} #{args}`
    end

    def git(command, *args)
      debug "git #{command} #{args * ' '}"
      sh :git, command, *args
    end

    def sh(*cmd)
      result = `#{cmd * ' '}`.chomp

      if $?.success?
        result
      elsif $-e
        exit 1
      else
        nil
      end
    end

    def exited_successfully?
      $!.nil? || $!.is_a?(SystemExit) && $!.success?
    end

    # Obtain a mutually exclusive lock to operate on a path safely
    def synchronize(path)
      require 'tempfile'
      path = File.join(Dir.tmpdir, "#{Rip.md5(path)}.lock")
      file = File.new(path, 'w+')
      file.flock(File::LOCK_EX)
      yield
    ensure
      file.flock(File::LOCK_UN)
      file.close
      # We can't safely cleanup the lock file. This litters tmp with
      # lock files. Not a big deal but we could do better.
    end
  end
end
